@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2016 Savoir-faire Linux Inc.
+ *  Copyright (C) 2004-2018 Savoir-faire Linux Inc.
  *
  *  Author: Thibault Wittemberg <thibault.wittemberg@savoirfairelinux.com>
  *
@@ -22,15 +22,15 @@ package cx.ring.dependencyinjection;
 import javax.inject.Singleton;
 
 import cx.ring.about.AboutFragment;
-import cx.ring.about.AboutPresenter;
 import cx.ring.account.AccountEditionActivity;
+import cx.ring.account.AccountWizardActivity;
 import cx.ring.account.HomeAccountCreationFragment;
+import cx.ring.account.ProfileCreationFragment;
 import cx.ring.account.RegisterNameDialog;
+import cx.ring.account.RingAccountCreationFragment;
 import cx.ring.account.RingAccountSummaryFragment;
-import cx.ring.account.RingAccountSummaryPresenter;
 import cx.ring.account.RingLinkAccountFragment;
 import cx.ring.application.RingApplication;
-import cx.ring.client.AccountWizard;
 import cx.ring.client.HomeActivity;
 import cx.ring.contactrequests.BlackListFragment;
 import cx.ring.contactrequests.ContactRequestsFragment;
@@ -42,13 +42,11 @@ import cx.ring.fragments.CallFragment;
 import cx.ring.fragments.ConversationFragment;
 import cx.ring.fragments.GeneralAccountFragment;
 import cx.ring.fragments.MediaPreferenceFragment;
-import cx.ring.account.ProfileCreationFragment;
-import cx.ring.account.RingAccountCreationFragment;
 import cx.ring.fragments.SIPAccountCreationFragment;
 import cx.ring.fragments.SecurityAccountFragment;
 import cx.ring.fragments.SmartListFragment;
+import cx.ring.launch.LaunchActivity;
 import cx.ring.navigation.RingNavigationFragment;
-import cx.ring.navigation.RingNavigationPresenter;
 import cx.ring.service.BootReceiver;
 import cx.ring.service.DRingService;
 import cx.ring.services.AccountService;
@@ -63,13 +61,24 @@ import cx.ring.services.NotificationServiceImpl;
 import cx.ring.services.PresenceService;
 import cx.ring.services.SharedPreferencesServiceImpl;
 import cx.ring.settings.SettingsFragment;
-import cx.ring.settings.SettingsPresenter;
 import cx.ring.share.ShareFragment;
-import cx.ring.share.SharePresenter;
+import cx.ring.tv.account.TVAccountExport;
+import cx.ring.tv.account.TVAccountWizard;
+import cx.ring.tv.account.TVHomeAccountCreationFragment;
+import cx.ring.tv.account.TVProfileCreationFragment;
+import cx.ring.tv.account.TVProfileEditingFragment;
+import cx.ring.tv.account.TVRingAccountCreationFragment;
+import cx.ring.tv.account.TVRingLinkAccountFragment;
+import cx.ring.tv.account.TVSettingsFragment;
+import cx.ring.tv.call.TVCallActivity;
+import cx.ring.tv.call.TVCallFragment;
+import cx.ring.tv.contactrequest.TVContactRequestFragment;
+import cx.ring.tv.main.MainFragment;
+import cx.ring.tv.search.RingSearchFragment;
 import dagger.Component;
 
 @Singleton
-@Component(modules = {RingInjectionModule.class, PresenterInjectionModule.class, ServiceInjectionModule.class})
+@Component(modules = {RingInjectionModule.class, ServiceInjectionModule.class})
 public interface RingInjectionComponent {
     void inject(RingApplication app);
 
@@ -77,7 +86,7 @@ public interface RingInjectionComponent {
 
     void inject(HomeActivity activity);
 
-    void inject(AccountWizard activity);
+    void inject(AccountWizardActivity activity);
 
     void inject(AccountEditionActivity activity);
 
@@ -143,16 +152,6 @@ public interface RingInjectionComponent {
 
     void inject(BootReceiver receiver);
 
-    void inject(AboutPresenter presenter);
-
-    void inject(RingNavigationPresenter presenter);
-
-    void inject(SharePresenter presenter);
-
-    void inject(SettingsPresenter presenter);
-
-    void inject(RingAccountSummaryPresenter presenter);
-
     void inject(AdvancedAccountFragment fragment);
 
     void inject(GeneralAccountFragment fragment);
@@ -160,4 +159,37 @@ public interface RingInjectionComponent {
     void inject(HomeAccountCreationFragment fragment);
 
     void inject(RingLinkAccountFragment fragment);
+
+    void inject(LaunchActivity activity);
+
+    //    AndroidTV section
+    void inject(TVCallFragment fragment);
+
+    void inject(MainFragment fragment);
+
+    void inject(RingSearchFragment fragment);
+
+    void inject(cx.ring.tv.main.HomeActivity activity);
+
+    void inject(TVCallActivity activity);
+
+    void inject(TVAccountWizard activity);
+
+    void inject(TVHomeAccountCreationFragment fragment);
+
+    void inject(TVProfileCreationFragment fragment);
+
+    void inject(TVRingAccountCreationFragment fragment);
+
+    void inject(TVRingLinkAccountFragment fragment);
+
+    void inject(TVAccountExport fragment);
+
+    void inject(TVProfileEditingFragment activity);
+
+    void inject(TVContactRequestFragment fragment);
+
+    void inject(TVSettingsFragment tvSettingsFragment);
+
+    void inject(TVSettingsFragment.PrefsFragment prefsFragment);
 }
