@@ -20,6 +20,7 @@
  */
 package cx.ring.services;
 
+import android.annotation.SuppressLint;
 import android.app.Notification;
 import android.app.NotificationChannel;
 import android.app.NotificationManager;
@@ -62,20 +63,20 @@ import cx.ring.model.Conference;
 import cx.ring.model.Conversation;
 import cx.ring.model.DataTransfer;
 import cx.ring.model.DataTransferEventCode;
-import cx.ring.model.ServiceEvent;
 import cx.ring.model.SipCall;
 import cx.ring.model.TextMessage;
 import cx.ring.model.TrustRequest;
 import cx.ring.model.Uri;
 import cx.ring.service.DRingService;
 import cx.ring.utils.BitmapUtils;
+import cx.ring.utils.FileUtils;
 import cx.ring.utils.Log;
-import cx.ring.utils.Observable;
-import cx.ring.utils.Observer;
+import cx.ring.utils.ResourceMapper;
 import ezvcard.VCard;
 import ezvcard.property.Photo;
+import io.reactivex.disposables.CompositeDisposable;
 
-public class NotificationServiceImpl extends NotificationService implements Observer<ServiceEvent> {
+public class NotificationServiceImpl extends NotificationService {
 
     private static final String TAG = NotificationServiceImpl.class.getSimpleName();
 
@@ -94,26 +95,32 @@ public class NotificationServiceImpl extends NotificationService implements Obse
     @Inject
     protected AccountService mAccountService;
     @Inject
-    protected CallService mCallService;
-    @Inject
     protected PreferencesService mPreferencesService;
     @Inject
     protected HistoryService mHistoryService;
     @Inject
     protected DeviceRuntimeService mDeviceRuntimeService;
     private NotificationManagerCompat notificationManager;
-    private Random random;
+    private final Random random = new Random();
 
+    private final CompositeDisposable mDisposable = new CompositeDisposable();
+
+    @SuppressLint("CheckResult")
     public void initHelper() {
         if (notificationManager == null) {
             notificationManager = NotificationManagerCompat.from(mContext);
         }
-        mAccountService.addObserver(this);
-        mCallService.addObserver(this);
         if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
             registerNotificationChannels();
         }
-        random = new Random();
+        mAccountService.getCurrentAccountSubject()
+                .switchMap(a -> a
+                        .getPendingSubject()
+                        .map(p -> {
+                            showIncomingTrustRequestNotification(a);
+                            return a;
+                        }))
+                .subscribe();
     }
 
     @RequiresApi(api = Build.VERSION_CODES.O)
@@ -177,7 +184,7 @@ public class NotificationServiceImpl extends NotificationService implements Obse
             messageNotificationBuilder.setContentTitle(mContext.getString(R.string.notif_current_call_title, contact.getRingUsername()))
                     .setContentText(mContext.getText(R.string.notif_current_call))
                     .setContentIntent(gotoIntent)
-                    .addAction(R.drawable.ic_call_end_white, mContext.getText(R.string.action_call_hangup),
+                    .addAction(R.drawable.ic_call_end_white_24dp, mContext.getText(R.string.action_call_hangup),
                             PendingIntent.getService(mContext, random.nextInt(),
                                     new Intent(DRingService.ACTION_CALL_END)
                                             .setClass(mContext, DRingService.class)
@@ -191,7 +198,7 @@ public class NotificationServiceImpl extends NotificationService implements Obse
                         .setContentText(mContext.getText(R.string.notif_incoming_call))
                         .setContentIntent(gotoIntent)
                         .setFullScreenIntent(gotoIntent, true)
-                        .addAction(R.drawable.ic_call_end_white, mContext.getText(R.string.action_call_decline),
+                        .addAction(R.drawable.ic_call_end_white_24dp, mContext.getText(R.string.action_call_decline),
                                 PendingIntent.getService(mContext, random.nextInt(),
                                         new Intent(DRingService.ACTION_CALL_REFUSE)
                                                 .setClass(mContext, DRingService.class)
@@ -208,7 +215,7 @@ public class NotificationServiceImpl extends NotificationService implements Obse
                 messageNotificationBuilder.setContentTitle(mContext.getString(R.string.notif_outgoing_call_title, contact.getRingUsername()))
                         .setContentText(mContext.getText(R.string.notif_outgoing_call))
                         .setContentIntent(gotoIntent)
-                        .addAction(R.drawable.ic_call_end_white, mContext.getText(R.string.action_call_hangup),
+                        .addAction(R.drawable.ic_call_end_white_24dp, mContext.getText(R.string.action_call_hangup),
                                 PendingIntent.getService(mContext, random.nextInt(),
                                         new Intent(DRingService.ACTION_CALL_END)
                                                 .setClass(mContext, DRingService.class)
@@ -232,26 +239,29 @@ public class NotificationServiceImpl extends NotificationService implements Obse
 
     @Override
     public void showTextNotification(CallContact contact, Conversation conversation, TreeMap<Long, TextMessage> texts) {
+        Uri contactUri = contact.getPrimaryUri();
         if (texts.isEmpty()) {
-            cancelTextNotification(contact);
+            cancelTextNotification(contactUri);
             return;
         }
         TextMessage last = texts.lastEntry().getValue();
+        String contactId = contactUri.getRawUriString();
 
         Intent intentConversation = new Intent(DRingService.ACTION_CONV_ACCEPT)
                 .setClass(mContext, DRingService.class)
                 .putExtra(ConversationFragment.KEY_ACCOUNT_ID, conversation.getLastAccountUsed())
-                .putExtra(ConversationFragment.KEY_CONTACT_RING_ID, contact.getPhones().get(0).getNumber().toString());
+                .putExtra(ConversationFragment.KEY_CONTACT_RING_ID, contactId);
 
         Intent intentDelete = new Intent(DRingService.ACTION_CONV_DISMISS)
                 .setClass(mContext, DRingService.class)
                 .putExtra(ConversationFragment.KEY_ACCOUNT_ID, conversation.getLastAccountUsed())
-                .putExtra(ConversationFragment.KEY_CONTACT_RING_ID, contact.getPhones().get(0).getNumber().toString());
+                .putExtra(ConversationFragment.KEY_CONTACT_RING_ID, contactId);
 
         NotificationCompat.Builder messageNotificationBuilder = new NotificationCompat.Builder(mContext, NOTIF_CHANNEL_MESSAGE)
                 .setCategory(NotificationCompat.CATEGORY_MESSAGE)
                 .setPriority(Notification.PRIORITY_HIGH)
                 .setDefaults(NotificationCompat.DEFAULT_ALL)
+                .setVisibility(NotificationCompat.VISIBILITY_PUBLIC)
                 .setSmallIcon(R.drawable.ic_ring_logo_white)
                 .setContentTitle(contact.getDisplayName())
                 .setContentText(last.getMessage())
@@ -286,16 +296,16 @@ public class NotificationServiceImpl extends NotificationService implements Obse
         Intent intentRead = new Intent(DRingService.ACTION_CONV_READ)
                 .setClass(mContext, DRingService.class)
                 .putExtra(ConversationFragment.KEY_ACCOUNT_ID, conversation.getLastAccountUsed())
-                .putExtra(ConversationFragment.KEY_CONTACT_RING_ID, contact.getPhones().get(0).getNumber().toString());
+                .putExtra(ConversationFragment.KEY_CONTACT_RING_ID, contactId);
 
         messageNotificationBuilder.addAction(0, mContext.getString(R.string.notif_mark_as_read), PendingIntent.getService(mContext, Long.valueOf(System.currentTimeMillis()).intValue(), intentRead, 0));
-        int notificationId = getTextNotificationId(contact);
+        int notificationId = getTextNotificationId(contactUri);
         notificationManager.notify(notificationId, messageNotificationBuilder.build());
         mNotificationBuilders.put(notificationId, messageNotificationBuilder);
     }
 
     @Override
-    public void showIncomingTrustRequestNotification(Account account) {
+    public void showIncomingTrustRequestNotification(final Account account) {
         int notificationId = getIncomingTrustNotificationId(account.getAccountID());
         NotificationCompat.Builder messageNotificationBuilder = mNotificationBuilders.get(notificationId);
 
@@ -305,23 +315,32 @@ public class NotificationServiceImpl extends NotificationService implements Obse
             messageNotificationBuilder = new NotificationCompat.Builder(mContext, NOTIF_CHANNEL_REQUEST);
         }
 
-        Collection<TrustRequest> requests = account.getRequests();
+        Set<String> notifiedRequests = mPreferencesService.loadRequestsPreferences(account.getAccountID());
+
+        Collection<Conversation> requests = account.getPending();
+        Log.w(TAG, "showIncomingTrustRequestNotification " + requests.size());
         if (requests.isEmpty()) {
             return;
         } else if (requests.size() == 1) {
-            TrustRequest request = requests.iterator().next();
+            Conversation request = requests.iterator().next();
+            CallContact contact = request.getContact();
+            String contactKey = contact.getKey();
+            if (notifiedRequests.contains(contactKey)) {
+                return;
+            }
+            mPreferencesService.saveRequestPreferences(account.getAccountID(), contactKey);
             messageNotificationBuilder = new NotificationCompat.Builder(mContext, NOTIF_CHANNEL_REQUEST);
             Bundle info = new Bundle();
             info.putString(TRUST_REQUEST_NOTIFICATION_ACCOUNT_ID, account.getAccountID());
-            info.putString(TRUST_REQUEST_NOTIFICATION_FROM, request.getContactId());
-            messageNotificationBuilder.setContentText(request.getDisplayname())
+            info.putString(TRUST_REQUEST_NOTIFICATION_FROM, contact.getPrimaryNumber());
+            messageNotificationBuilder.setContentText(contact.getRingUsername())
                     .addAction(R.drawable.ic_action_accept, mContext.getText(R.string.accept),
                             PendingIntent.getService(mContext, random.nextInt(),
                                     new Intent(DRingService.ACTION_TRUST_REQUEST_ACCEPT)
                                             .setClass(mContext, DRingService.class)
                                             .putExtras(info),
                                     PendingIntent.FLAG_ONE_SHOT))
-                    .addAction(R.drawable.ic_delete_white, mContext.getText(R.string.refuse),
+                    .addAction(R.drawable.ic_delete_white_24dp, mContext.getText(R.string.refuse),
                             PendingIntent.getService(mContext, random.nextInt(),
                                     new Intent(DRingService.ACTION_TRUST_REQUEST_REFUSE)
                                             .setClass(mContext, DRingService.class)
@@ -335,14 +354,24 @@ public class NotificationServiceImpl extends NotificationService implements Obse
                                     PendingIntent.FLAG_ONE_SHOT));
 
 
-            VCard vCard = request.getVCard();
-            List<Photo> photos = vCard == null ? null : vCard.getPhotos();
+            List<Photo> photos = contact.vcard == null ? null : contact.vcard.getPhotos();
             byte[] data = null;
             if (photos != null && !photos.isEmpty()) {
                 data = photos.get(0).getData();
             }
-            setContactPicture(data, request.getDisplayname(), request.getContactId(), messageNotificationBuilder);
+            setContactPicture(data, contact.getRingUsername(), contact.getPrimaryNumber(), messageNotificationBuilder);
         } else {
+            boolean newRequest = false;
+            for (Conversation request : requests) {
+                CallContact contact = request.getContact();
+                String contactKey = contact.getKey();
+                if (!notifiedRequests.contains(contactKey)) {
+                    newRequest = true;
+                    mPreferencesService.saveRequestPreferences(account.getAccountID(), contactKey);
+                }
+            }
+            if (!newRequest)
+                return;
             messageNotificationBuilder.setContentText(String.format(mContext.getString(R.string.contact_request_msg), Integer.toString(requests.size())));
             messageNotificationBuilder.setLargeIcon(null);
             messageNotificationBuilder.mActions.clear();
@@ -351,6 +380,7 @@ public class NotificationServiceImpl extends NotificationService implements Obse
         messageNotificationBuilder
                 .setDefaults(NotificationCompat.DEFAULT_ALL)
                 .setPriority(NotificationCompat.PRIORITY_HIGH)
+                .setVisibility(NotificationCompat.VISIBILITY_PUBLIC)
                 .setAutoCancel(true)
                 .setSmallIcon(R.drawable.ic_ring_logo_white)
                 .setCategory(NotificationCompat.CATEGORY_SOCIAL)
@@ -369,8 +399,12 @@ public class NotificationServiceImpl extends NotificationService implements Obse
     }
 
     @Override
-    public void showFileTransferNotification(DataTransfer info, DataTransferEventCode event, CallContact contact) {
-        if (event == null || info == null) {
+    public void showFileTransferNotification(DataTransfer info, CallContact contact) {
+        if (info == null) {
+            return;
+        }
+        DataTransferEventCode event = info.getEventCode();
+        if (event == null) {
             return;
         }
         long dataTransferId = info.getDataTransferId();
@@ -399,12 +433,16 @@ public class NotificationServiceImpl extends NotificationService implements Obse
         Bitmap photo_bmp = photo == null ? null : BitmapFactory.decodeByteArray(photo, 0, photo.length);
         messageNotificationBuilder.setContentTitle(titleMessage)
                 .setPriority(NotificationCompat.PRIORITY_DEFAULT)
+                .setVisibility(NotificationCompat.VISIBILITY_PUBLIC)
                 .setAutoCancel(false)
                 .setOngoing(ongoing)
                 .setSmallIcon(R.drawable.ic_ring_logo_white)
                 .setLargeIcon(photo_bmp)
                 .setCategory(NotificationCompat.CATEGORY_PROGRESS)
-                .setContentText(info.getDisplayName() + " state: " + event.name())
+                .setOnlyAlertOnce(true)
+                .setContentText(event == DataTransferEventCode.ONGOING ?
+                        FileUtils.readableFileProgress(info.getBytesProgress(), info.getTotalSize()) :
+                        info.getDisplayName() + ": " + ResourceMapper.getReadableFileTransferStatus(mContext, event))
                 .setContentIntent(PendingIntent.getService(mContext, random.nextInt(), intentConversation, 0))
                 .setColor(ResourcesCompat.getColor(mContext.getResources(), R.color.color_primary_dark, null));
         if (event.isOver()) {
@@ -423,13 +461,13 @@ public class NotificationServiceImpl extends NotificationService implements Obse
         messageNotificationBuilder.mActions.clear();
         if (event == DataTransferEventCode.WAIT_HOST_ACCEPTANCE) {
             messageNotificationBuilder
-                    .addAction(R.drawable.ic_incoming_black, mContext.getText(R.string.accept),
+                    .addAction(R.drawable.ic_call_received_black_24dp, mContext.getText(R.string.accept),
                     PendingIntent.getService(mContext, random.nextInt(),
                             new Intent(DRingService.ACTION_FILE_ACCEPT)
                                     .setClass(mContext, DRingService.class)
                                     .putExtra(DRingService.KEY_TRANSFER_ID, dataTransferId),
                             PendingIntent.FLAG_ONE_SHOT))
-                    .addAction(R.drawable.ic_cancel_black_24dp, mContext.getText(R.string.refuse),
+                    .addAction(R.drawable.baseline_cancel_black_24, mContext.getText(R.string.refuse),
                             PendingIntent.getService(mContext, random.nextInt(),
                                     new Intent(DRingService.ACTION_FILE_CANCEL)
                                             .setClass(mContext, DRingService.class)
@@ -437,7 +475,7 @@ public class NotificationServiceImpl extends NotificationService implements Obse
                                     PendingIntent.FLAG_ONE_SHOT));
         } else if (!event.isOver()) {
             messageNotificationBuilder
-                    .addAction(R.drawable.ic_cancel_black_24dp, mContext.getText(android.R.string.cancel),
+                    .addAction(R.drawable.baseline_cancel_black_24, mContext.getText(android.R.string.cancel),
                             PendingIntent.getService(mContext, random.nextInt(),
                                     new Intent(DRingService.ACTION_FILE_CANCEL)
                                             .setClass(mContext, DRingService.class)
@@ -450,7 +488,7 @@ public class NotificationServiceImpl extends NotificationService implements Obse
     }
 
     @Override
-    public void cancelTextNotification(CallContact contact) {
+    public void cancelTextNotification(Uri contact) {
         if (contact == null) {
             return;
         }
@@ -491,8 +529,8 @@ public class NotificationServiceImpl extends NotificationService implements Obse
         return (NOTIF_TRUST_REQUEST + accountId).hashCode();
     }
 
-    private int getTextNotificationId(CallContact contact) {
-        return (NOTIF_MSG + contact.getPhones().get(0).getNumber().toString()).hashCode();
+    private int getTextNotificationId(Uri contact) {
+        return (NOTIF_MSG + contact.toString()).hashCode();
     }
 
     private int getFileTransferNotificationId(Long dataTransferId) {
@@ -519,26 +557,4 @@ public class NotificationServiceImpl extends NotificationService implements Obse
         }
         messageNotificationBuilder.setLargeIcon(circleBitmap);
     }
-
-    @Override
-    public void update(Observable observable, ServiceEvent event) {
-        if (observable instanceof AccountService && event != null) {
-            switch (event.getEventType()) {
-                case INCOMING_TRUST_REQUEST: {
-                    final String accountID = event.getEventInput(ServiceEvent.EventInput.ACCOUNT_ID, String.class);
-                    final String from = event.getEventInput(ServiceEvent.EventInput.FROM, String.class);
-                    Log.d(TAG, "update: INCOMING_TRUST_REQUEST " + accountID + " " + from);
-                    Account account = mAccountService.getAccount(accountID);
-                    Set<String> notifiedRequests = mPreferencesService.loadRequestsPreferences(accountID);
-                    if (notifiedRequests == null || !notifiedRequests.contains(from)) {
-                        showIncomingTrustRequestNotification(account);
-                        mPreferencesService.saveRequestPreferences(accountID, from);
-                    } else {
-                        Log.d(TAG, "INCOMING_TRUST_REQUEST: already notified for " + from);
-                    }
-                    break;
-                }
-            }
-        }
-    }
 }
\ No newline at end of file
